@@ -381,6 +381,9 @@ def canonicalish(schema: JSONType) -> Dict[str, Any]:
         "maxProperties", math.inf
     ):
         type_.remove("object")
+    # Remove no-op requires
+    if "required" in schema and not schema["required"]:
+        schema.pop("required")
     # Canonicalise "required" schemas to remove redundancy
     if "object" in type_ and "required" in schema:
         assert isinstance(schema["required"], list)
@@ -402,10 +405,10 @@ def canonicalish(schema: JSONType) -> Dict[str, Any]:
         schema["required"] = sorted(reqs)
         max_ = schema.get("maxProperties", float("inf"))
         assert isinstance(max_, (int, float))
-        propnames = schema.get("propertyNames", {})
         if len(schema["required"]) > max_:
             type_.remove("object")
         else:
+            propnames = schema.get("propertyNames", {})
             validator = make_validator(propnames)
             if not all(validator.is_valid(name) for name in schema["required"]):
                 type_.remove("object")
@@ -416,9 +419,7 @@ def canonicalish(schema: JSONType) -> Dict[str, Any]:
             continue
         for k in kw.split():
             schema.pop(k, None)
-    # Remove no-op requires
-    if "required" in schema and not schema["required"]:
-        schema.pop("required")
+
     # Canonicalise "not" subschemas
     if "not" in schema:
         not_ = schema.pop("not")
@@ -584,18 +585,19 @@ def merged(schemas: List[Any]) -> Optional[Schema]:
     It's currently also used for keys that could be merged but aren't yet.
     """
     assert schemas, "internal error: must pass at least one schema to merge"
-    out = canonicalish(schemas[0])
+    schemas = sorted((canonicalish(s) for s in schemas), key=upper_bound_instances)
+    if any(s == FALSEY for s in schemas):
+        return FALSEY
+    out = schemas[0]
     for s in schemas[1:]:
-        s = canonicalish(s)
         # If we have a const or enum, this is fairly easy by filtering:
-        if "const" in s:
-            if make_validator(out).is_valid(s["const"]):
-                out = s
+        if "const" in out:
+            if make_validator(s).is_valid(out["const"]):
                 continue
             return FALSEY
-        if "enum" in s:
-            validator = make_validator(out)
-            enum_ = [v for v in s["enum"] if validator.is_valid(v)]
+        if "enum" in out:
+            validator = make_validator(s)
+            enum_ = [v for v in out["enum"] if validator.is_valid(v)]
             if not enum_:
                 return FALSEY
             elif len(enum_) == 1:
@@ -606,7 +608,9 @@ def merged(schemas: List[Any]) -> Optional[Schema]:
 
         if "type" in out and "type" in s:
             tt = s.pop("type")
-            out["type"] = [t for t in get_type(out) if t in tt]
+            out["type"] = [
+                t for t in get_type(out) if t in tt or t == "integer" and "number" in tt
+            ]
             if not get_type(out):
                 return FALSEY
             for t, kw in TYPE_SPECIFIC_KEYS:
@@ -689,6 +693,7 @@ def merged(schemas: List[Any]) -> Optional[Schema]:
         out = canonicalish(out)
         if out == FALSEY:
             return FALSEY
+    assert isinstance(out, dict)
     jsonschema.validators.validator_for(out).check_schema(out)
     return out
 
