@@ -5,8 +5,7 @@ import math
 import operator
 import re
 from fractions import Fraction
-from functools import partial
-from typing import Any, Callable, Dict, List, Set, Union
+from typing import Any, Callable, Dict, List, Optional, Set, Union
 
 import hypothesis.internal.conjecture.utils as cu
 import hypothesis.provisional as prov
@@ -27,7 +26,7 @@ from ._canonicalise import (
     get_integer_bounds,
     get_number_bounds,
     get_type,
-    is_valid,
+    make_validator,
     merged,
     resolve_all_refs,
 )
@@ -58,8 +57,11 @@ def merged_as_strategies(schemas: List[Schema]) -> st.SearchStrategy[JSONType]:
             continue
         s = merged([inputs[g] for g in group])
         if s is not None and s != FALSEY:
+            validators = [make_validator(s) for s in schemas]
             strats.append(
-                from_schema(s).filter(lambda v: all(is_valid(v, s) for s in schemas))
+                from_schema(s).filter(
+                    lambda obj: all(v.is_valid(obj) for v in validators)
+                )
             )
             combined.update(group)
     return st.one_of(strats)
@@ -95,7 +97,8 @@ def from_schema(schema: Union[bool, Schema]) -> st.SearchStrategy[JSONType]:
     if "not" in schema:
         not_ = schema.pop("not")
         assert isinstance(not_, dict)
-        return from_schema(schema).filter(lambda v: not is_valid(v, not_))
+        validator = make_validator(not_).is_valid
+        return from_schema(schema).filter(lambda v: not validator(v))
     if "anyOf" in schema:
         tmp = schema.copy()
         ao = tmp.pop("anyOf")
@@ -112,7 +115,7 @@ def from_schema(schema: Union[bool, Schema]) -> st.SearchStrategy[JSONType]:
         assert isinstance(oo, list)
         schemas = [merged([tmp, s]) for s in oo]
         return st.one_of([from_schema(s) for s in schemas if s is not None]).filter(
-            partial(is_valid, schema=schema)
+            make_validator(schema).is_valid
         )
     # Conditional application of subschemata
     if "if" in schema:
@@ -124,7 +127,7 @@ def from_schema(schema: Union[bool, Schema]) -> st.SearchStrategy[JSONType]:
         assert isinstance(then, (bool, dict))
         assert isinstance(else_, (bool, dict))
         return st.one_of([from_schema(s) for s in (then, else_, if_, tmp)]).filter(
-            partial(is_valid, schema=schema)
+            make_validator(schema).is_valid
         )
     # Simple special cases
     if "enum" in schema:
@@ -146,40 +149,42 @@ def from_schema(schema: Union[bool, Schema]) -> st.SearchStrategy[JSONType]:
     return st.one_of([map_[t](schema) for t in get_type(schema)])
 
 
+def _numeric_with_multiplier(
+    min_value: Optional[float], max_value: Optional[float], schema: Schema
+) -> st.SearchStrategy[float]:
+    """Handle numeric schemata containing the multipleOf key."""
+    multiple_of = schema["multipleOf"]
+    assert isinstance(multiple_of, (int, float))
+    if min_value is not None:
+        min_value = math.ceil(Fraction(min_value) / Fraction(multiple_of))
+    if max_value is not None:
+        max_value = math.floor(Fraction(max_value) / Fraction(multiple_of))
+    if min_value is not None and max_value is not None and min_value > max_value:
+        # You would think that this is impossible, but it can happen if multipleOf
+        # is very small and the bounds are very close togther.  It would be nicer
+        # to deal with this when canonicalising, but suffice to say we can't without
+        # diverging from the floating-point behaviour of the upstream validator.
+        return st.nothing()
+    return (
+        st.integers(min_value, max_value)
+        .map(lambda x: x * multiple_of)
+        .filter(make_validator(schema).is_valid)
+    )
+
+
 def integer_schema(schema: dict) -> st.SearchStrategy[float]:
     """Handle integer schemata."""
-    # TODO: possibly generate value as a float if float(x) == x
     min_value, max_value = get_integer_bounds(schema)
-
     if "multipleOf" in schema:
-        multiple_of = schema["multipleOf"]
-        assert isinstance(multiple_of, (int, float))
-        if min_value is not None:
-            min_value = math.ceil(Fraction(min_value) / Fraction(multiple_of))
-        if max_value is not None:
-            max_value = math.floor(Fraction(max_value) / Fraction(multiple_of))
-        strat = st.integers(min_value, max_value).map(lambda x: x * multiple_of)
-        # check for and filter out float bounds, inexact multiplication, etc.
-        return strat.filter(partial(is_valid, schema=schema))
-
+        return _numeric_with_multiplier(min_value, max_value, schema)
     return st.integers(min_value, max_value)
 
 
 def number_schema(schema: dict) -> st.SearchStrategy[float]:
     """Handle numeric schemata."""
     min_value, max_value, exclude_min, exclude_max = get_number_bounds(schema)
-
     if "multipleOf" in schema:
-        multiple_of = schema["multipleOf"]
-        assert isinstance(multiple_of, (int, float))
-        if min_value is not None:
-            min_value = math.ceil(Fraction(min_value) / Fraction(multiple_of))
-        if max_value is not None:
-            max_value = math.floor(Fraction(max_value) / Fraction(multiple_of))
-        strat = st.integers(min_value, max_value).map(lambda x: x * multiple_of)
-        # check for and filter out float bounds, inexact multiplication, etc.
-        return strat.filter(partial(is_valid, schema=schema))
-
+        return _numeric_with_multiplier(min_value, max_value, schema)
     return st.floats(
         min_value=min_value,
         max_value=max_value,
@@ -191,39 +196,6 @@ def number_schema(schema: dict) -> st.SearchStrategy[float]:
     ).filter(lambda n: n != 0 or math.copysign(1, n) == 1)
 
 
-RFC3339_FORMATS = (
-    "date-fullyear",
-    "date-month",
-    "date-mday",
-    "time-hour",
-    "time-minute",
-    "time-second",
-    "time-secfrac",
-    "time-numoffset",
-    "time-offset",
-    "partial-time",
-    "full-date",
-    "full-time",
-    "date-time",
-)
-JSON_SCHEMA_STRING_FORMATS = RFC3339_FORMATS + (
-    "email",
-    "idn-email",
-    "hostname",
-    "idn-hostname",
-    "ipv4",
-    "ipv6",
-    "uri",
-    "uri-reference",
-    "iri",
-    "iri-reference",
-    "uri-template",
-    "json-pointer",
-    "relative-json-pointer",
-    "regex",
-)
-
-
 def rfc3339(name: str) -> st.SearchStrategy[str]:
     """Get a strategy for date or time strings in the given RFC3339 format.
 
@@ -255,9 +227,9 @@ def rfc3339(name: str) -> st.SearchStrategy[str]:
         return st.one_of(st.just("Z"), rfc3339("time-numoffset"))
     if name == "partial-time":
         return st.times().map(str)
-    if name == "full-date":
+    if name == "date" or name == "full-date":
         return st.dates().map(str)
-    if name == "full-time":
+    if name == "time" or name == "full-time":
         return st.tuples(rfc3339("partial-time"), rfc3339("time-offset")).map("".join)
     assert name == "date-time"
     return st.tuples(rfc3339("full-date"), rfc3339("full-time")).map("T".join)
@@ -284,10 +256,47 @@ def regex_patterns(draw: Any) -> str:
 
 REGEX_PATTERNS = regex_patterns()
 
+# Via the `webcolors` package, to match the logic `jsonschema`
+# uses to check it's (non-standard?) "color" format.
+_WEBCOLOR_REGEX = "^#([a-fA-F0-9]{3}|[a-fA-F0-9]{6})$"
+_CSS21_COLOR_NAMES = (
+    "aqua",
+    "black",
+    "blue",
+    "fuchsia",
+    "green",
+    "gray",
+    "lime",
+    "maroon",
+    "navy",
+    "olive",
+    "orange",
+    "purple",
+    "red",
+    "silver",
+    "teal",
+    "white",
+    "yellow",
+)
+
+RFC3339_FORMATS = (
+    "date-fullyear",
+    "date-month",
+    "date-mday",
+    "time-hour",
+    "time-minute",
+    "time-second",
+    "time-secfrac",
+    "time-numoffset",
+    "time-offset",
+    "partial-time",
+    "full-date",
+    "full-time",
+    "date-time",
+)
 STRING_FORMATS = {
     **{name: rfc3339(name) for name in RFC3339_FORMATS},
-    "date": rfc3339("full-date"),
-    "time": rfc3339("full-time"),
+    "color": st.from_regex(_WEBCOLOR_REGEX) | st.sampled_from(_CSS21_COLOR_NAMES),
     "email": st.emails(),
     "idn-email": st.emails(),
     "hostname": prov.domains(),
@@ -308,8 +317,8 @@ def string_schema(schema: dict) -> st.SearchStrategy[str]:
     """Handle schemata for strings."""
     # also https://json-schema.org/latest/json-schema-validation.html#rfc.section.7
     min_size = schema.get("minLength", 0)
-    max_size = schema.get("maxLength", float("inf"))
-    strategy = st.text(min_size=min_size, max_size=schema.get("maxLength"))
+    max_size = schema.get("maxLength")
+    strategy = st.text(min_size=min_size, max_size=max_size)
     if schema.get("format") in STRING_FORMATS:
         # Unknown "format" specifiers should be ignored for validation.
         # See https://json-schema.org/latest/json-schema-validation.html#format
@@ -326,7 +335,14 @@ def string_schema(schema: dict) -> st.SearchStrategy[str]:
         except re.error:
             # Patterns that are invalid in Python, or just malformed
             return st.nothing()
-    return strategy.filter(lambda s: min_size <= len(s) <= max_size)
+    # If we have size bounds but we're generating strings from a regex or pattern,
+    # apply a filter to ensure our size bounds are respected.
+    if ("format" in schema or "pattern" in schema) and (
+        min_size != 0 or max_size is not None
+    ):
+        max_size = math.inf if max_size is None else max_size
+        strategy = strategy.filter(lambda s: min_size <= len(s) <= max_size)
+    return strategy
 
 
 def array_schema(schema: dict) -> st.SearchStrategy[List[JSONType]]:
@@ -378,7 +394,8 @@ def array_schema(schema: dict) -> st.SearchStrategy[List[JSONType]]:
         )
     if "contains" not in schema:
         return strat
-    return strat.filter(lambda val: any(is_valid(x, schema["contains"]) for x in val))
+    contains = make_validator(schema["contains"]).is_valid
+    return strat.filter(lambda val: any(contains(x) for x in val))
 
 
 def object_schema(schema: dict) -> st.SearchStrategy[Dict[str, JSONType]]:
@@ -413,7 +430,7 @@ def object_schema(schema: dict) -> st.SearchStrategy[Dict[str, JSONType]]:
         st.one_of([st.from_regex(p) for p in sorted(patterns)]),
     )
     all_names_strategy = st.one_of([s for s in name_strats if not s.is_empty]).filter(
-        partial(is_valid, schema=names)
+        make_validator(names).is_valid
     )
 
     @st.composite  # type: ignore
@@ -461,7 +478,7 @@ def object_schema(schema: dict) -> st.SearchStrategy[Dict[str, JSONType]]:
                 out[key] = draw(from_schema(additional))
 
             for k, v in dep_schemas.items():
-                if k in out and not is_valid(out, v):
+                if k in out and not make_validator(v).is_valid(out):
                     out.pop(key)
                     elements.reject()
 
